"""
pons translator API
"""
from bs4 import BeautifulSoup
import requests
from .constants import BASE_URLS, PONS_LANGUAGES_TO_CODES, PONS_CODES_TO_LANGUAGES
from .exceptions import (LanguageNotSupportedException,
                        TranslationNotFound,
                        NotValidPayload,
                        ElementNotFoundInGetRequest,
                        RequestError,
                        TooManyRequests)
from .parent import BaseTranslator
from requests.utils import requote_uri


class PonsTranslator(BaseTranslator):
    """
    class that uses PONS translator to translate words
    """
    _languages = PONS_LANGUAGES_TO_CODES
    supported_languages = list(_languages.keys())

    def __init__(self, source, target="en", proxies=None, **kwargs):
        """
        @param source: source language to translate from
        @param target: target language to translate to
        """
        self.__base_url = BASE_URLS.get("PONS")
        self.proxies = proxies
        if self.is_language_supported(source, target):
            self._source, self._target = self._map_language_to_code(source, target)

        super().__init__(base_url=self.__base_url,
                         source=self._source,
                         target=self._target,
                         payload_key=None,
                         element_tag='div',
                         element_query={"class": "target"}
                         )

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
          return the supported languages by the linguee translator
          @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
          @return: list or dict
          """
        return PonsTranslator.supported_languages if not as_dict else PonsTranslator._languages

    def _map_language_to_code(self, *languages, **kwargs):
        """
           map language to its corresponding code (abbreviation) if the language was passed by its full name by the user
           @param languages: list of languages
           @return: mapped value of the language or raise an exception if the language is not supported
        """
        for language in languages:
            if language in self._languages.values():
                yield PONS_CODES_TO_LANGUAGES[language]
            elif language in self._languages.keys():
                yield language
            else:
                raise LanguageNotSupportedException(language)

    def is_language_supported(self, *languages, **kwargs):
        """
         check if the language is supported by the translator
         @param languages: list of languages
         @return: bool or raise an Exception
         """
        for lang in languages:
            if lang not in self._languages.keys():
                if lang not in self._languages.values():
                    raise LanguageNotSupportedException(lang)
        return True

    def translate(self, word, return_all=False, **kwargs):
        """
        function that uses PONS to translate a word
        @param word: word to translate
        @type word: str
        @param return_all: set to True to return all synonym of the translated word
        @type return_all: bool
        @return: str: translated word
        """
        if self._validate_payload(word, max_chars=50):
            url = "{}{}-{}/{}".format(self.__base_url, self._source, self._target, word)
            url = requote_uri(url)
            response = requests.get(url, proxies=self.proxies)

            if response.status_code == 429:
                raise TooManyRequests()

            if response.status_code != 200:
                raise RequestError()

            soup = BeautifulSoup(response.text, 'html.parser')
            elements = soup.findAll(self._element_tag, self._element_query)

            if not elements:
                raise ElementNotFoundInGetRequest(word)

            filtered_elements = []
            for el in elements:
                temp = ''
                for e in el.findAll('a'):
                    if e.parent.name == 'div':
                        if e and "/translate/{}-{}/".format(self._target, self._source) in e.get('href'):
                            temp += e.get_text() + ' '
                filtered_elements.append(temp)

            if not filtered_elements:
                raise ElementNotFoundInGetRequest(word)

            word_list = [word for word in filtered_elements if word and len(word) > 1]

            if not word_list:
                raise TranslationNotFound(word)

            return word_list if return_all else word_list[0]

    def translate_words(self, words, **kwargs):
        """
        translate a batch of words together by providing them in a list
        @param words: list of words you want to translate
        @param kwargs: additional args
        @return: list of translated words
        """
        if not words:
            raise NotValidPayload(words)

        translated_words = []
        for word in words:
            translated_words.append(self.translate(word=word, **kwargs))
        return translated_words




"""parent translator class"""

from .exceptions import NotValidPayload, NotValidLength, InvalidSourceOrTargetLanguage
from abc import ABC, abstractmethod
import string


class BaseTranslator(ABC):
    """
    Abstract class that serve as a parent translator for other different translators
    """
    def __init__(self,
                 base_url=None,
                 source="auto",
                 target="en",
                 payload_key=None,
                 element_tag=None,
                 element_query=None,
                 **url_params):
        """
        @param source: source language to translate from
        @param target: target language to translate to
        """
        if source == target:
            raise InvalidSourceOrTargetLanguage(source)

        self.__base_url = base_url
        self._source = source
        self._target = target
        self._url_params = url_params
        self._element_tag = element_tag
        self._element_query = element_query
        self.payload_key = payload_key
        self.headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) '
                                      'AppleWebit/535.19'
                                      '(KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19'}
        super(BaseTranslator, self).__init__()

    @staticmethod
    def _validate_payload(payload, min_chars=1, max_chars=5000):
        """
        validate the target text to translate
        @param payload: text to translate
        @return: bool
        """

        if not payload or not isinstance(payload, str) or not payload.strip() or payload.isdigit():
            raise NotValidPayload(payload)

        # check if payload contains only symbols
        if all(i in string.punctuation for i in payload):
            raise NotValidPayload(payload)

        if not BaseTranslator.__check_length(payload, min_chars, max_chars):
            raise NotValidLength(payload, min_chars, max_chars)
        return True

    @staticmethod
    def __check_length(payload, min_chars, max_chars):
        """
        check length of the provided target text to translate
        @param payload: text to translate
        @param min_chars: minimum characters allowed
        @param max_chars: maximum characters allowed
        @return: bool
        """
        return True if min_chars <= len(payload) < max_chars else False

    @abstractmethod
    def translate(self, text, **kwargs):
        """
        translate a text using a translator under the hood and return the translated text
        @param text: text to translate
        @param kwargs: additional arguments
        @return: str
        """
        return NotImplemented('You need to implement the translate method!')






"""
Yandex translator API
"""
import requests
from .constants import BASE_URLS
from .exceptions import (RequestError, ServerException, TranslationNotFound, TooManyRequests)


class YandexTranslator(object):
    """
    class that wraps functions, which use the yandex translator under the hood to translate word(s)
    """

    def __init__(self, api_key=None, source="en", target="de", **kwargs):
        """
        @param api_key: your yandex api key
        """
        if not api_key:
            raise ServerException(401)
        self.__base_url = BASE_URLS.get("YANDEX")
        self.source = source
        self.target = target

        self.api_key = api_key
        self.api_version = "v1.5"
        self.api_endpoints = {
            "langs": "getLangs",
            "detect": "detect",
            "translate": "translate",
        }

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """ this method is just for consistency."""
        return """ this method is just for consistency. You need to create an instance of yandex and access
                    supported languages using the languages property or call _get_supported_languages
                """

    def _get_supported_languages(self):
        return set(x.split("-")[0] for x in self.dirs)

    @property
    def languages(self):
        return self.get_supported_languages()

    @property
    def dirs(self, proxies=None):

        try:
            url = self.__base_url.format(version=self.api_version, endpoint="getLangs")
            print("url: ", url)
            response = requests.get(url, params={"key": self.api_key}, proxies=proxies)
        except requests.exceptions.ConnectionError:
            raise ServerException(503)
        else:
            data = response.json()

        if response.status_code != 200:
            raise ServerException(response.status_code)
        return data.get("dirs")

    def detect(self, text, proxies=None):
        response = None
        params = {
            "text": text,
            "format": "plain",
            "key": self.api_key,
        }
        try:
            url = self.__base_url.format(version=self.api_version, endpoint="detect")
            response = requests.post(url, data=params, proxies=proxies)

        except RequestError:
            raise
        except ConnectionError:
            raise ServerException(503)
        except ValueError:
            raise ServerException(response.status_code)
        else:
            response = response.json()
        language = response['lang']
        status_code = response['code']
        if status_code != 200:
            raise RequestError()
        elif not language:
            raise ServerException(501)
        return language

    def translate(self, text, proxies=None, **kwargs):
        params = {
            "text": text,
            "format": "plain",
            "lang": self.target if self.source == "auto" else "{}-{}".format(self.source, self.target),
            "key": self.api_key
        }
        try:
            url = self.__base_url.format(version=self.api_version, endpoint="translate")
            response = requests.post(url, data=params, proxies=proxies)
        except ConnectionError:
            raise ServerException(503)
        else:
            response = response.json()

        if response['code'] == 429:
            raise TooManyRequests()

        if response['code'] != 200:
            raise ServerException(response['code'])

        if not response['text']:
            raise TranslationNotFound()

        return response['text']

    def translate_file(self, path, **kwargs):
        """
        translate from a file
        @param path: path to file
        @return: translated text
        """
        try:
            with open(path, 'r', encoding='utf-8') as f:
                text = f.read()

            return self.translate(text)
        except Exception as e:
            raise e

    def translate_batch(self, batch, **kwargs):
        """
        translate a batch of texts
        @param batch: list of texts to translate
        @return: list of translations
        """
        return [self.translate(text, **kwargs) for text in batch]



import requests

BASE_URLS = {
    "GOOGLE_TRANSLATE": "https://translate.google.com/m",
    "PONS": "https://en.pons.com/translate/",
    "YANDEX": "https://translate.yandex.net/api/{version}/tr.json/{endpoint}",
    "LINGUEE": "https://www.linguee.com/",
    "MYMEMORY": "http://api.mymemory.translated.net/get",
    "QCRI": "https://mt.qcri.org/api/v1/{endpoint}?",
    "DEEPL": "https://api.deepl.com/{version}/",
    "DEEPL_FREE": "https://api-free.deepl.com/v2/",
    "MICROSOFT_TRANSLATE": "https://api.cognitive.microsofttranslator.com/translate?api-version=3.0",
    "PAPAGO": "https://papago.naver.com/",
    "PAPAGO_API": "https://openapi.naver.com/v1/papago/n2mt",
    "LIBRE": "https://libretranslate.com/",
    "LIBRE_FREE": "https://libretranslate.de/",
}

GOOGLE_CODES_TO_LANGUAGES = {
    'af': 'afrikaans',
    'sq': 'albanian',
    'am': 'amharic',
    'ar': 'arabic',
    'hy': 'armenian',
    'az': 'azerbaijani',
    'eu': 'basque',
    'be': 'belarusian',
    'bn': 'bengali',
    'bs': 'bosnian',
    'bg': 'bulgarian',
    'ca': 'catalan',
    'ceb': 'cebuano',
    'ny': 'chichewa',
    'zh-CN': 'chinese (simplified)',
    'zh-TW': 'chinese (traditional)',
    'co': 'corsican',
    'hr': 'croatian',
    'cs': 'czech',
    'da': 'danish',
    'nl': 'dutch',
    'en': 'english',
    'eo': 'esperanto',
    'et': 'estonian',
    'tl': 'filipino',
    'fi': 'finnish',
    'fr': 'french',
    'fy': 'frisian',
    'gl': 'galician',
    'ka': 'georgian',
    'de': 'german',
    'el': 'greek',
    'gu': 'gujarati',
    'ht': 'haitian creole',
    'ha': 'hausa',
    'haw': 'hawaiian',
    'iw': 'hebrew',
    'hi': 'hindi',
    'hmn': 'hmong',
    'hu': 'hungarian',
    'is': 'icelandic',
    'ig': 'igbo',
    'id': 'indonesian',
    'ga': 'irish',
    'it': 'italian',
    'ja': 'japanese',
    'jw': 'javanese',
    'kn': 'kannada',
    'kk': 'kazakh',
    'km': 'khmer',
    'rw': 'kinyarwanda',
    'ko': 'korean',
    'ku': 'kurdish',
    'ky': 'kyrgyz',
    'lo': 'lao',
    'la': 'latin',
    'lv': 'latvian',
    'lt': 'lithuanian',
    'lb': 'luxembourgish',
    'mk': 'macedonian',
    'mg': 'malagasy',
    'ms': 'malay',
    'ml': 'malayalam',
    'mt': 'maltese',
    'mi': 'maori',
    'mr': 'marathi',
    'mn': 'mongolian',
    'my': 'myanmar',
    'ne': 'nepali',
    'no': 'norwegian',
    'or': 'odia',
    'ps': 'pashto',
    'fa': 'persian',
    'pl': 'polish',
    'pt': 'portuguese',
    'pa': 'punjabi',
    'ro': 'romanian',
    'ru': 'russian',
    'sm': 'samoan',
    'gd': 'scots gaelic',
    'sr': 'serbian',
    'st': 'sesotho',
    'sn': 'shona',
    'sd': 'sindhi',
    'si': 'sinhala',
    'sk': 'slovak',
    'sl': 'slovenian',
    'so': 'somali',
    'es': 'spanish',
    'su': 'sundanese',
    'sw': 'swahili',
    'sv': 'swedish',
    'tg': 'tajik',
    'ta': 'tamil',
    'tt': 'tatar',
    'te': 'telugu',
    'th': 'thai',
    'tr': 'turkish',
    'tk': 'turkmen',
    'uk': 'ukrainian',
    'ur': 'urdu',
    'ug': 'uyghur',
    'uz': 'uzbek',
    'vi': 'vietnamese',
    'cy': 'welsh',
    'xh': 'xhosa',
    'yi': 'yiddish',
    'yo': 'yoruba',
    'zu': 'zulu',
}

GOOGLE_LANGUAGES_TO_CODES = {v: k for k, v in GOOGLE_CODES_TO_LANGUAGES.items()}

# This dictionary maps the primary name of language to its secondary names in list manner (if any)
GOOGLE_LANGUAGES_SECONDARY_NAMES = {
    'myanmar': ['burmese'],
    'odia': ['oriya'],
    'kurdish':  ['kurmanji']
}


PONS_CODES_TO_LANGUAGES = {
    'ar': 'arabic',
    'bg': 'bulgarian',
    'zh-cn': 'chinese',
    'cs': 'czech',
    'da': 'danish',
    'nl': 'dutch',
    'en': 'english',
    'fr': 'french',
    'de': 'german',
    'el': 'greek',
    'hu': 'hungarian',
    'it': 'italian',
    'la': 'latin',
    'no': 'norwegian',
    'pl': 'polish',
    'pt': 'portuguese',
    'ru': 'russian',
    'sl': 'slovenian',
    'es': 'spanish',
    'sv': 'swedish',
    'tr': 'turkish',
    'elv': 'elvish'
}

PONS_LANGUAGES_TO_CODES = {v: k for k, v in PONS_CODES_TO_LANGUAGES.items()}

LINGUEE_LANGUAGES_TO_CODES = {
    "maltese": "mt",
    "english": "en",
    "german": "de",
    "bulgarian": "bg",
    "polish": "pl",
    "portuguese": "pt",
    "hungarian": "hu",
    "romanian": "ro",
    "russian": "ru",
    # "serbian": "sr",
    "dutch": "nl",
    "slovakian": "sk",
    "greek": "el",
    "slovenian": "sl",
    "danish": "da",
    "italian": "it",
    "spanish": "es",
    "finnish": "fi",
    "chinese": "zh",
    "french": "fr",
    # "croatian": "hr",
    "czech": "cs",
    "laotian": "lo",
    "swedish": "sv",
    "latvian": "lv",
    "estonian": "et",
    "japanese": "ja"
}

LINGUEE_CODE_TO_LANGUAGE = {v: k for k, v in LINGUEE_LANGUAGES_TO_CODES.items()}

# "72e9e2cc7c992db4dcbdd6fb9f91a0d1"

# obtaining the current list of supported Microsoft languages for translation

microsoft_languages_api_url = "https://api.cognitive.microsofttranslator.com/languages?api-version=3.0&scope=translation"
microsoft_languages_response = requests.get(microsoft_languages_api_url)
translation_dict = microsoft_languages_response.json()['translation']

MICROSOFT_CODES_TO_LANGUAGES = {translation_dict[k]['name'].lower(): k for k in translation_dict.keys()}

DEEPL_LANGUAGE_TO_CODE = {
    "bulgarian": "bg",
    "czech": "cs",
    "danish": "da",
    "german": "de",
    "greek": "el",
    "english": "en",
    "spanish": "es",
    "estonian": "et",
    "finnish": "fi",
    "french": "fr",
    "hungarian": "hu",
    "italian": "it",
    "japanese": "ja",
    "lithuanian": "lt",
    "latvian": "lv",
    "dutch": "nl",
    "polish": "pl",
    "portuguese": "pt",
    "romanian": "ro",
    "russian": "ru",
    "slovak": "sk",
    "slovenian": "sl",
    "swedish": "sv",
    "chinese": "zh"
}

DEEPL_CODE_TO_LANGUAGE = {v: k for k, v in DEEPL_LANGUAGE_TO_CODE.items()}

PAPAGO_CODE_TO_LANGUAGE = {
    'ko': 'Korean',
    'en': 'English',
    'ja': 'Japanese',
    'zh-CN': 'Chinese',
    'zh-TW': 'Chinese traditional',
    'es': 'Spanish',
    'fr': 'French',
    'vi': 'Vietnamese',
    'th': 'Thai',
    'id': 'Indonesia'
}

PAPAGO_LANGUAGE_TO_CODE = {v: k for v, k in PAPAGO_CODE_TO_LANGUAGE.items()}

QCRI_CODE_TO_LANGUAGE = {
    'ar': 'Arabic',
    'en': 'English',
    'es': 'Spanish'
}

QCRI_LANGUAGE_TO_CODE = {
    v: k for k, v in QCRI_CODE_TO_LANGUAGE.items()
}

LIBRE_CODES_TO_LANGUAGES = {
    'en': 'English',
    'ar': 'Arabic',
    'zh': 'Chinese',
    'fr': 'French',
    'de': 'German',
    'hi': 'Hindi',
    'id': 'Indonesian',
    'ga': 'Irish',
    'it': 'Italian',
    'ja': 'Japanese',
    'ko': 'Korean',
    'pl': 'Polish',
    'pt': 'Portuguese',
    'ru': 'Russian',
    'es': 'Spanish',
    'tr': 'Turkish',
    'vi': 'Vietnamese'
}

LIBRE_LANGUAGES_TO_CODES = {
    v: k for k, v in LIBRE_CODES_TO_LANGUAGES.items()
}



"""Console script for deep_translator."""

import click
from .google_trans import GoogleTranslator
from .mymemory import MyMemoryTranslator
from .deepl import DeepL
from .qcri import QCRI
from .linguee import LingueeTranslator
from .pons import PonsTranslator
from .yandex import YandexTranslator
from .microsoft import MicrosoftTranslator
from .papago import PapagoTranslator
from .libre import LibreTranslator

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
@click.group()
def cli():
    pass

@cli.command(context_settings=CONTEXT_SETTINGS, no_args_is_help=True)
@click.argument('translator', required=True, default='google', type=str)
@click.option("--source", "-src", required=True, type=str, help="source language to translate from")
@click.option("--target", "-tgt", required=True, type=str, help="target language to translate to")
@click.option("--text", "-txt", type=str,required = True,prompt="Enter the text you want to translate",help="text you want to translate")
@click.option("--api-key",type=str,help="required for DeepL, QCRI, Yandex, Microsoft and Papago translators")
def translate(translator, source, target, text, api_key):
    """
    Use TRANSLATOR to translate source material into another language.
    \f
    Directory function to send arguments to the correct translator.
    @param translator: translator name parsed from terminal arguments
    @return: None
    """
    api_key_required = ["deepl", "qcri", "yandex", "microsoft", "papago"]
    if translator in api_key_required and not api_key:
        click.echo(
            "This translator requires an api key provided through --api-key")
    else:
        pass

    if translator == "google":
        translator = GoogleTranslator(source=source, target=target)
    elif translator == "mymemory":
        translator = MyMemoryTranslator(source=source, target=target)
    elif translator == "deepl":
        translator = DeepL(source=source, target=target, api_key=api_key)
    elif translator == "qcri":
        translator = QCRI(source=source, target=target, api_key=api_key)
    elif translator == "linguee":
        translator = LingueeTranslator(source=source, target=target)
    elif translator == "pons":
        translator = PonsTranslator(source=source, target=target)
    elif translator == "yandex":
        translator = YandexTranslator(
            source=source,
            target=target,
            api_key=api_key)
    elif translator == "microsoft":
        translator = MicrosoftTranslator(
            source=source,
            target=target,
            api_key=api_key)
    elif translator == "papago":
        translator = PapagoTranslator(
            source=source,
            target=target,
            api_key=api_key)
    elif translator == "libre":
        translator= LibreTranslator(
            source=source,
            target=target
        )
    else:
        raise AttributeError("The given translator is not supported.")

    res = translator.translate(text)
    click.echo(f" | Translation from {source} to {target} |")
    click.echo(f"Translated text: \n {res}")
    return 0

@cli.command(context_settings=CONTEXT_SETTINGS, no_args_is_help=True)
@click.argument('translator')
@click.argument('api_key', required=False)
def languages(translator, api_key):
    """
    Retrieve the list of available languages from the given translator.
    @param translator: Translator given by the user.
    @param api_key: Optional API key given by the user. Required for some translators.
    @return: None
    """
    translator = translator.lower()
    api_key_required = ["deepl", "qcri", "yandex", "microsoft", "papago"]
    if translator in api_key_required and not api_key:
        click.echo("This translator requires an api key provided through --api-key")
    else:
        pass

    if translator == "google":
        translator = GoogleTranslator
    elif translator == "mymemory":
        translator = MyMemoryTranslator
    elif translator == "qcri":
        translator = QCRI(api_key=api_key)
    elif translator == "linguee":
        translator = LingueeTranslator
    elif translator == "pons":
        translator = PonsTranslator
    elif translator == "yandex":
        translator = YandexTranslator(api_key=api_key)
    elif translator == "microsoft":
        translator = MicrosoftTranslator(api_key=api_key)
    elif translator == "papago":
        translator = PapagoTranslator(api_key=api_key)
    elif translator == "libre":
        translator = LibreTranslator
    else:
        raise AttributeError("The given translator is not supported.")

    supported_languages = translator.get_supported_languages(as_dict=True)
    click.echo(f"Languages supported by '{translator}' are :")
    for k, v in supported_languages.items():
        click.echo(f"|- {k}: {v}")
    return 0

@cli.command()
def list():
    """Lists available translators."""
    click.echo("Available translators include: Google, MyMemory, QCRI, Linguee, Pons, Yandex, Microsoft (Bing), Papago and LibreTranslate.")
    return 0

if __name__ == "__main__":
    cli()



"""Top-level package for Deep Translator"""

from .google_trans import GoogleTranslator
from .pons import PonsTranslator
from .linguee import LingueeTranslator
from .mymemory import MyMemoryTranslator
from .yandex import YandexTranslator
from .qcri import QCRI
from .deepl import DeepL
from .detection import single_detection, batch_detection
from .microsoft import MicrosoftTranslator
from .papago import PapagoTranslator
from .libre import LibreTranslator

__author__ = """Nidhal Baccouri"""
__email__ = 'nidhalbacc@gmail.com'
__version__ = '1.6.1'

__all__ = [
    "GoogleTranslator",
    "PonsTranslator",
    "LingueeTranslator",
    "MyMemoryTranslator",
    "YandexTranslator",
    "MicrosoftTranslator",
    "QCRI",
    "DeepL",
    "LibreTranslator",
    "PapagoTranslator",
    "single_detection",
    "batch_detection"
    ]



"""
LibreTranslate API
"""

import requests
from .parent import BaseTranslator
from .constants import BASE_URLS,LIBRE_LANGUAGES_TO_CODES, LIBRE_CODES_TO_LANGUAGES
from .exceptions import (ServerException,
                        TranslationNotFound,
                        LanguageNotSupportedException,
                        AuthorizationException,
                        NotValidPayload)


class LibreTranslator(BaseTranslator):
    """
    class that wraps functions, which use libre translator under the hood to translate text(s)
    """
    _languages = LIBRE_LANGUAGES_TO_CODES
    supported_languages = list(_languages.keys())

    def __init__(self,source="auto", target="en", base_url = BASE_URLS.get("LIBRE_FREE"), api_key=None, **kwargs):
        """
        @param source: source language to translate from
        List of LibreTranslate nedpoints can be found at : https://github.com/LibreTranslate/LibreTranslate#mirrors
        Some require an API key
        @param target: target language to translate to
        """
        if base_url == BASE_URLS.get("LIBRE") and not api_key:
            raise ServerException(401)
        self.__base_url = base_url
        self.api_key = api_key
        if source == "auto":
            self.source = "auto"
        else:
            self.source = self._map_language_to_code(source)
        self.target = self._map_language_to_code(target)


    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
        return the supported languages by the libre translator
        @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
        @return: list or dict
        """
        return [*LibreTranslator._languages.keys()] if not as_dict else LibreTranslator._languages

    def _map_language_to_code(self, language, **kwargs):
        """
        map language to its corresponding code (abbreviation) if the language was passed by its full name by the user
        @param language: a string for 1 language
        @return: mapped value of the language or raise an exception if the language is not supported
        """
        if language in self._languages.keys():
            return self._languages[language]
        elif language in self._languages.values():
            return language
        raise LanguageNotSupportedException(language)

    def _is_language_supported(self, language, **kwargs):
        """
        check if the language is supported by the translator
        @param language: a string for 1 language
        @return: bool or raise an Exception
        """
        if language == 'auto' or language in self._languages.keys() or language in self._languages.values():
            return True
        else:
            raise LanguageNotSupportedException(language)

    def translate(self, text, **kwargs):
        """
        function that uses microsoft translate to translate a text
        @param text: desired text to translate
        @return: str: translated text
        """
         # Create the request parameters.
        if type(text) != str or text == "":
            raise NotValidPayload(text)

        translate_endpoint = 'translate'
        params = {
            "q": text,
            "source": self.source,
            "target": self.target,
            "format": 'text'
        }
        # Add API Key if required
        if self.api_key:
            params["api_key"] = self.api_key
        # Do the request and check the connection.
        try:
            response = requests.post(self.__base_url + translate_endpoint, params=params)
        except ConnectionError:
            raise ServerException(503)
        # If the answer is not success, raise server exception.

        if response.status_code == 403:
            raise AuthorizationException(self.api_key)
        elif response.status_code != 200:
            raise ServerException(response.status_code)
        # Get the response and check is not empty.
        res = response.json()
        if not res:
            raise TranslationNotFound(text)
        # Process and return the response.
        return res['translatedText']

    def translate_file(self, path, **kwargs):
        """
        translate directly from file
        @param path: path to the target file
        @type path: str
        @param kwargs: additional args
        @return: str
        """
        try:
            with open(path, 'r', encoding='utf-8') as f:
                text = f.read().strip()
            return self.translate(text)
        except Exception as e:
            raise e

    def translate_batch(self, batch=None, **kwargs):
        """
        translate a list of texts
        @param batch: list of texts you want to translate
        @return: list of translations
        """
        if not batch:
            raise Exception("Enter your text list that you want to translate")
        arr = []
        for i, text in enumerate(batch):
            translated = self.translate(text, **kwargs)
            arr.append(translated)
        return arr



"""
linguee translator API
"""

from .constants import BASE_URLS, LINGUEE_LANGUAGES_TO_CODES, LINGUEE_CODE_TO_LANGUAGE
from .exceptions import (LanguageNotSupportedException,
                        TranslationNotFound,
                        NotValidPayload,
                        ElementNotFoundInGetRequest,
                        RequestError,
                        TooManyRequests)
from .parent import BaseTranslator
from bs4 import BeautifulSoup
import requests
from requests.utils import requote_uri


class LingueeTranslator(BaseTranslator):
    """
    class that wraps functions, which use the linguee translator under the hood to translate word(s)
    """
    _languages = LINGUEE_LANGUAGES_TO_CODES
    supported_languages = list(_languages.keys())

    def __init__(self, source, target="en", proxies=None, **kwargs):
        """
        @param source: source language to translate from
        @param target: target language to translate to
        """
        self.__base_url = BASE_URLS.get("LINGUEE")
        self.proxies = proxies

        if self.is_language_supported(source, target):
            self._source, self._target = self._map_language_to_code(source.lower(), target.lower())

        super().__init__(base_url=self.__base_url,
                         source=self._source,
                         target=self._target,
                         element_tag='a',
                         element_query={'class': 'dictLink featured'},
                         payload_key=None,  # key of text in the url
                        )

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
        return the supported languages by the linguee translator
        @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
        @return: list or dict
        """
        return LingueeTranslator.supported_languages if not as_dict else LingueeTranslator._languages

    def _map_language_to_code(self, *languages, **kwargs):
        """
         map language to its corresponding code (abbreviation) if the language was passed by its full name by the user
         @param languages: list of languages
         @return: mapped value of the language or raise an exception if the language is not supported
         """
        for language in languages:
            if language in self._languages.values():
                yield LINGUEE_CODE_TO_LANGUAGE[language]
            elif language in self._languages.keys():
                yield language
            else:
                raise LanguageNotSupportedException(language)

    def is_language_supported(self, *languages, **kwargs):
        """
        check if the language is supported by the translator
        @param languages: list of languages
        @return: bool or raise an Exception
        """
        for lang in languages:
            if lang not in self._languages.keys():
                if lang not in self._languages.values():
                    raise LanguageNotSupportedException(lang)
        return True

    def translate(self, word, return_all=False, **kwargs):
        """
        function that uses linguee to translate a word
        @param word: word to translate
        @type word: str
        @param return_all: set to True to return all synonym of the translated word
        @type return_all: bool
        @return: str: translated word
        """
        if self._validate_payload(word, max_chars=50):
            # %s-%s/translation/%s.html
            url = "{}{}-{}/translation/{}.html".format(self.__base_url, self._source, self._target, word)
            url = requote_uri(url)
            response = requests.get(url, proxies=self.proxies)

            if response.status_code == 429:
                raise TooManyRequests()

            if response.status_code != 200:
                raise RequestError()
            soup = BeautifulSoup(response.text, 'html.parser')
            elements = soup.find_all(self._element_tag, self._element_query)
            if not elements:
                raise ElementNotFoundInGetRequest(elements)

            filtered_elements = []
            for el in elements:
                try:
                    pronoun = el.find('span', {'class': 'placeholder'}).get_text(strip=True)
                except AttributeError:
                    pronoun = ''
                filtered_elements.append(el.get_text(strip=True).replace(pronoun, ''))

            if not filtered_elements:
                raise TranslationNotFound(word)

            return filtered_elements if return_all else filtered_elements[0]

    def translate_words(self, words, **kwargs):
        """
        translate a batch of words together by providing them in a list
        @param words: list of words you want to translate
        @param kwargs: additional args
        @return: list of translated words
        """
        if not words:
            raise NotValidPayload(words)

        translated_words = []
        for word in words:
            translated_words.append(self.translate(word=word, **kwargs))
        return translated_words




"""
google translator API
"""
import json
from .constants import BASE_URLS, PAPAGO_LANGUAGE_TO_CODE
from .exceptions import LanguageNotSupportedException, TranslationNotFound, NotValidPayload
import requests
import warnings
import logging


class PapagoTranslator(object):
    """
    class that wraps functions, which use google translate under the hood to translate text(s)
    """
    _languages = PAPAGO_LANGUAGE_TO_CODE
    supported_languages = list(_languages.keys())

    def __init__(self, client_id=None, secret_key=None, source="auto", target="en", **kwargs):
        """
        @param source: source language to translate from
        @param target: target language to translate to
        """
        if not client_id or not secret_key:
            raise Exception("Please pass your client id and secret key! visit the papago website for more infos")

        self.__base_url = BASE_URLS.get("PAPAGO_API")
        self.client_id = client_id
        self.secret_key = secret_key
        if self.is_language_supported(source, target):
            self._source, self._target = self._map_language_to_code(source.lower(), target.lower())

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
        return the supported languages by the google translator
        @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
        @return: list or dict
        """
        return PapagoTranslator.supported_languages if not as_dict else PapagoTranslator._languages

    def _map_language_to_code(self, *languages):
        """
        map language to its corresponding code (abbreviation) if the language was passed by its full name by the user
        @param languages: list of languages
        @return: mapped value of the language or raise an exception if the language is not supported
        """
        for language in languages:
            if language in self._languages.values() or language == 'auto':
                yield language
            elif language in self._languages.keys():
                yield self._languages[language]
            else:
                raise LanguageNotSupportedException(language)

    def is_language_supported(self, *languages):
        """
        check if the language is supported by the translator
        @param languages: list of languages
        @return: bool or raise an Exception
        """
        for lang in languages:
            if lang != 'auto' and lang not in self._languages.keys():
                if lang != 'auto' and lang not in self._languages.values():
                    raise LanguageNotSupportedException(lang)
        return True

    def translate(self, text, **kwargs):
        """
        function that uses google translate to translate a text
        @param text: desired text to translate
        @return: str: translated text
        """

        payload = {
            "source": self._source,
            "target": self._target,
            "text": text
        }
        headers = {
            'X-Naver-Client-Id': self.client_id,
            'X-Naver-Client-Secret': self.secret_key,
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
        }
        response = requests.post(self.__base_url, headers=headers, data=payload)
        if response.status_code != 200:
            raise Exception(f'Translation error! -> status code: {response.status_code}')
        res_body = json.loads(response.text)
        if "message" not in res_body:
            raise TranslationNotFound(text)

        msg = res_body.get("message")
        result = msg.get("result", None)
        if not result:
            raise TranslationNotFound(text)
        translated_text = result.get("translatedText")
        return translated_text

    def translate_file(self, path, **kwargs):
        """
        translate directly from file
        @param path: path to the target file
        @type path: str
        @param kwargs: additional args
        @return: str
        """
        try:
            with open(path, 'r', encoding='utf-8') as f:
                text = f.read().strip()
            return self.translate(text)
        except Exception as e:
            raise e

    def translate_sentences(self, sentences=None, **kwargs):
        """
        translate many sentences together. This makes sense if you have sentences with different languages
        and you want to translate all to unified language. This is handy because it detects
        automatically the language of each sentence and then translate it.

        @param sentences: list of sentences to translate
        @return: list of all translated sentences
        """
        warnings.warn("deprecated. Use the translate_batch function instead", DeprecationWarning, stacklevel=2)
        logging.warning("deprecated. Use the translate_batch function instead")
        if not sentences:
            raise NotValidPayload(sentences)

        translated_sentences = []
        try:
            for sentence in sentences:
                translated = self.translate(text=sentence)
                translated_sentences.append(translated)

            return translated_sentences

        except Exception as e:
            raise e

    def translate_batch(self, batch=None, **kwargs):
        """
        translate a list of texts
        @param batch: list of texts you want to translate
        @return: list of translations
        """
        if not batch:
            raise Exception("Enter your text list that you want to translate")
        arr = []
        for i, text in enumerate(batch):

            translated = self.translate(text, **kwargs)
            arr.append(translated)
        return arr






import requests
from .constants import BASE_URLS, QCRI_LANGUAGE_TO_CODE
from .exceptions import (ServerException, TranslationNotFound)


class QCRI(object):
    """
    class that wraps functions, which use the QRCI translator under the hood to translate word(s)
    """

    def __init__(self, api_key=None, source="en", target="en", **kwargs):
        """
        @param api_key: your qrci api key. Get one for free here https://mt.qcri.org/api/v1/ref
        """

        if not api_key:
            raise ServerException(401)
        self.__base_url = BASE_URLS.get("QCRI")
        self.source = source
        self.target = target
        self.api_key = api_key
        self.api_endpoints = {
            "get_languages": "getLanguagePairs",
            "get_domains": "getDomains",
            "translate": "translate",
        }

        self.params = {
            "key": self.api_key
        }

    def _get(self, endpoint, params=None, return_text=True):
        if not params:
            params = self.params
        try:
            res = requests.get(self.__base_url.format(endpoint=self.api_endpoints[endpoint]), params=params)
            return res.text if return_text else res
        except Exception as e:
            raise e

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        # Have no use for this as the format is not what we need
        # Save this for whenever
        # pairs = self._get("get_languages")
        # Using a this one instead
        return [*QCRI_LANGUAGE_TO_CODE.keys()] if not as_dict else QCRI_LANGUAGE_TO_CODE

    @property
    def languages(self):
        return self.get_supported_languages()

    def get_domains(self):
        domains = self._get("get_domains")
        return domains

    @property
    def domains(self):
        return self.get_domains()

    def translate(self, text, domain, **kwargs):
        params = {
            "key": self.api_key,
            "langpair": "{}-{}".format(self.source, self.target),
            "domain": domain,
            "text": text
        }
        try:
            response = self._get("translate", params=params, return_text=False)
        except ConnectionError:
            raise ServerException(503)

        else:
            if response.status_code != 200:
                ServerException(response.status_code)
            else:
                res = response.json()
                translation = res.get("translatedText")
                if not translation:
                    raise TranslationNotFound(text)
                return translation

    def translate_batch(self, batch, domain, **kwargs):
        """
        translate a batch of texts
        @domain: domain
        @param batch: list of texts to translate
        @return: list of translations
        """
        return [self.translate(domain, text, **kwargs) for text in batch]




"""
google translator API
"""

from .constants import BASE_URLS, GOOGLE_LANGUAGES_TO_CODES, GOOGLE_LANGUAGES_SECONDARY_NAMES
from .exceptions import TooManyRequests, LanguageNotSupportedException, TranslationNotFound, NotValidPayload, RequestError
from .parent import BaseTranslator
from bs4 import BeautifulSoup
import requests
from time import sleep
import warnings
import logging


class GoogleTranslator(BaseTranslator):
    """
    class that wraps functions, which use google translate under the hood to translate text(s)
    """
    _languages = GOOGLE_LANGUAGES_TO_CODES
    supported_languages = list(_languages.keys())

    def __init__(self, source="auto", target="en", proxies=None, **kwargs):
        """
        @param source: source language to translate from
        @param target: target language to translate to
        """
        self.__base_url = BASE_URLS.get("GOOGLE_TRANSLATE")
        self.proxies = proxies

        # code snipppet that converts the language into lower-case and skip lower-case conversion for abbreviations
        # since abbreviations like zh-CN if converted to lower-case will result into error
        #######################################
        source_lower = source
        target_lower = target
        if not source in self._languages.values():
            source_lower=source.lower()
        if not target in self._languages.values():
            target_lower=target.lower()
        #######################################

        if self.is_language_supported(source_lower, target_lower):
            self._source, self._target = self._map_language_to_code(source_lower, target_lower)

        super(GoogleTranslator, self).__init__(base_url=self.__base_url,
                                               source=self._source,
                                               target=self._target,
                                               element_tag='div',
                                               element_query={"class": "t0"},
                                               payload_key='q',  # key of text in the url
                                               tl=self._target,
                                               sl=self._source,
                                               **kwargs)

        self._alt_element_query = {"class": "result-container"}

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
        return the supported languages by the google translator
        @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
        @return: list or dict
        """
        return GoogleTranslator.supported_languages if not as_dict else GoogleTranslator._languages

    def is_secondary(self, lang):
        """
        Function to check if lang is a secondary name of any primary language
        @param lang: language name
        @return: primary name of a language if found otherwise False
        """
        for primary_name, secondary_names in GOOGLE_LANGUAGES_SECONDARY_NAMES.items():
            if lang in secondary_names:
                return primary_name
        return False

    def _map_language_to_code(self, *languages):
        """
        map language to its corresponding code (abbreviation) if the language was passed by its full name by the user
        @param languages: list of languages
        @return: mapped value of the language or raise an exception if the language is not supported
        """
        for language in languages:
            if language in self._languages.values() or language == 'auto':
                yield language
            elif language in self._languages.keys():
                yield self._languages[language]
            else:
                yield self._languages[self.is_secondary(language)]

    def is_language_supported(self, *languages):
        """
        check if the language is supported by the translator
        @param languages: list of languages
        @return: bool or raise an Exception
        """
        for lang in languages:
            if lang != 'auto' and lang not in self._languages.keys():
                if lang != 'auto' and lang not in self._languages.values():
                    if not self.is_secondary(lang):
                        raise LanguageNotSupportedException(lang)
        return True

    def translate(self, text, **kwargs):
        """
        function that uses google translate to translate a text
        @param text: desired text to translate
        @return: str: translated text
        """

        if self._validate_payload(text):
            text = text.strip()

            if self.payload_key:
                self._url_params[self.payload_key] = text

            response = requests.get(self.__base_url,
                                    params=self._url_params,
                                    proxies=self.proxies)
            if response.status_code == 429:
                raise TooManyRequests()

            if response.status_code != 200:
                raise RequestError()

            soup = BeautifulSoup(response.text, 'html.parser')

            element = soup.find(self._element_tag, self._element_query)

            if not element:
                element = soup.find(self._element_tag, self._alt_element_query)
                if not element:
                    raise TranslationNotFound(text)
            if element.get_text(strip=True) == text.strip():
                to_translate_alpha = ''.join(ch for ch in text.strip() if ch.isalnum())
                translated_alpha = ''.join(ch for ch in element.get_text(strip=True) if ch.isalnum())
                if to_translate_alpha and translated_alpha and to_translate_alpha == translated_alpha:
                    self._url_params["tl"] = self._target
                    if "hl" not in self._url_params:
                        return text.strip()
                    del self._url_params["hl"]
                    return self.translate(text)

            else:
                return element.get_text(strip=True)

    def translate_file(self, path, **kwargs):
        """
        translate directly from file
        @param path: path to the target file
        @type path: str
        @param kwargs: additional args
        @return: str
        """
        try:
            with open(path, 'r', encoding='utf-8') as f:
                text = f.read().strip()
            return self.translate(text)
        except Exception as e:
            raise e

    def translate_sentences(self, sentences=None, **kwargs):
        """
        translate many sentences together. This makes sense if you have sentences with different languages
        and you want to translate all to unified language. This is handy because it detects
        automatically the language of each sentence and then translate it.

        @param sentences: list of sentences to translate
        @return: list of all translated sentences
        """
        warnings.warn("deprecated. Use the translate_batch function instead", DeprecationWarning, stacklevel=2)
        logging.warning("deprecated. Use the translate_batch function instead")
        if not sentences:
            raise NotValidPayload(sentences)

        translated_sentences = []
        try:
            for sentence in sentences:
                translated = self.translate(text=sentence)
                translated_sentences.append(translated)

            return translated_sentences

        except Exception as e:
            raise e

    def translate_batch(self, batch=None, **kwargs):
        """
        translate a list of texts
        @param batch: list of texts you want to translate
        @return: list of translations
        """
        if not batch:
            raise Exception("Enter your text list that you want to translate")
        arr = []
        for i, text in enumerate(batch):

            translated = self.translate(text, **kwargs)
            arr.append(translated)
        return arr




"""
language detection API
"""
import requests
from requests.exceptions import HTTPError

# Module global config
config = {"url": 'https://ws.detectlanguage.com/0.2/detect',"headers": {'User-Agent': 'Detect Language API Python Client 1.4.0','Authorization': 'Bearer {}',}}

def get_request_body(text, api_key, *args, **kwargs):
    """
    send a request and return the response body parsed as dictionary

    @param text: target text that you want to detect its language
    @type text: str
    @type api_key: str
    @param api_key: your private API key

    """
    if not api_key:
        raise Exception("you need to get an API_KEY for this to work. "
                        "Get one for free here: https://detectlanguage.com/documentation")
    if not text:
        raise Exception("Please provide an input text")

    else:
        try:
            headers = config['headers']
            headers['Authorization'] = headers['Authorization'].format(api_key)
            response = requests.post(config['url'],
                                     json={'q': text},
                                     headers=headers)

            body = response.json().get('data')
            return body

        except HTTPError as e:
            print("Error occured while requesting from server: ", e.args)
            raise e


def single_detection(text, api_key=None, detailed=False, *args, **kwargs):
    """
    function responsible for detecting the language from a text

    @param text: target text that you want to detect its language
    @type text: str
    @type api_key: str
    @param api_key: your private API key
    @param detailed: set to True if you want to get detailed information about the detection process
    """
    body = get_request_body(text, api_key)
    detections = body.get('detections')
    if detailed:
        return detections[0]

    lang = detections[0].get('language', None)
    if lang:
        return lang


def batch_detection(text_list, api_key, detailed=False, *args, **kwargs):
    """
    function responsible for detecting the language from a text

    @param text_list: target batch that you want to detect its language
    @param api_key: your private API key
    @param detailed: set to True if you want to get detailed information about the detection process
    """
    body = get_request_body(text_list, api_key)
    detections = body.get('detections')
    res = [obj[0] for obj in detections]
    if detailed:
        return res
    else:
        return [obj['language'] for obj in res]




class BaseError(Exception):
    """
    base error structure class
    """

    def __init__(self, val, message):
        """
        @param val: actual value
        @param message: message shown to the user
        """
        self.val = val
        self.message = message
        super().__init__()

    def __str__(self):
        return "{} --> {}".format(self.val, self.message)


class LanguageNotSupportedException(BaseError):
    """
    exception thrown if the user uses a language that is not supported by the deep_translator
    """

    def __init__(self, val, message="There is no support for the chosen language"):
        super().__init__(val, message)


class NotValidPayload(BaseError):
    """
    exception thrown if the user enters an invalid payload
    """

    def __init__(self,
                 val,
                 message='text must be a valid text with maximum 5000 character, otherwise it cannot be translated'):
        super(NotValidPayload, self).__init__(val, message)


class InvalidSourceOrTargetLanguage(BaseError):
    """
    exception thrown if the user enters an invalid payload
    """

    def __init__(self,
                 val,
                 message="source and target language can't be the same"):
        super(InvalidSourceOrTargetLanguage, self).__init__(val, message)



class TranslationNotFound(BaseError):
    """
    exception thrown if no translation was found for the text provided by the user
    """

    def __init__(self,
                 val,
                 message='No translation was found using the current translator. Try another translator?'):
        super(TranslationNotFound, self).__init__(val, message)


class ElementNotFoundInGetRequest(BaseError):
    """
    exception thrown if the html element was not found in the body parsed by beautifulsoup
    """

    def __init__(self,
                 val,
                 message='Required element was not found in the API response'):
        super(ElementNotFoundInGetRequest, self).__init__(val, message)


class NotValidLength(BaseError):
    """
    exception thrown if the provided text exceed the length limit of the translator
    """

    def __init__(self, val, min_chars, max_chars):
        message = "Text length need to be between {} and {} characters".format(min_chars, max_chars)
        super(NotValidLength, self).__init__(val, message)


class RequestError(Exception):
    """
    exception thrown if an error occurred during the request call, e.g a connection problem.
    """

    def __init__(self, message="Request exception can happen due to an api connection error. "
                               "Please check your connection and try again"):
        self.message = message

    def __str__(self):
        return self.message


class MicrosoftAPIerror(Exception):
    """
    exception thrown if Microsoft API returns one of its errors
    """

    def __init__(self, api_message):
        self.api_message = str(api_message)
        self.message="Microsoft API returned the following error"

    def __str__(self):
        return "{}: {}".format(self.message, self.api_message)


class TooManyRequests(Exception):
    """
    exception thrown if an error occurred during the request call, e.g a connection problem.
    """

    def __init__(self, message="Server Error: You made too many requests to the server. According to google, you are allowed to make 5 requests per second and up to 200k requests per day. You can wait and try again later or you can try the translate_batch function"):
        self.message = message

    def __str__(self):
        return self.message


class ServerException(Exception):
    """
    Default YandexTranslate exception from the official website
    """
    errors = {
        400: "ERR_BAD_REQUEST",
        401: "ERR_KEY_INVALID",
        402: "ERR_KEY_BLOCKED",
        403: "ERR_DAILY_REQ_LIMIT_EXCEEDED",
        404: "ERR_DAILY_CHAR_LIMIT_EXCEEDED",
        413: "ERR_TEXT_TOO_LONG",
        429: "ERR_TOO_MANY_REQUESTS",
        422: "ERR_UNPROCESSABLE_TEXT",
        500: "ERR_INTERNAL_SERVER_ERROR",
        501: "ERR_LANG_NOT_SUPPORTED",
        503: "ERR_SERVICE_NOT_AVAIBLE",
    }

    def __init__(self, status_code, *args):
        message = self.errors.get(status_code, "API server error")
        super(ServerException, self).__init__(message, *args)


class AuthorizationException(Exception):
    def __init__(self, api_key, *args):
        msg = 'Unauthorized access with the api key ' + api_key
        super().__init__(msg, *args)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deep_translator']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.1,<5.0.0',
 'click>=8.0.1,<9.0.0',
 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['deep-translator = deep_translator.main:cli',
                     'dt = deep_translator.main:cli']}

setup_kwargs = {
    'name': 'deep-translator',
    'version': '1.6.1',
    'description': 'A flexible free and unlimited python tool to translate between different languages in a simple way using multiple translators',
    'long_description': '##################\ndeep-translator\n##################\n\n.. image:: ../assets/icon.jpg\n    :width: 100\n    :align: center\n    :alt: deep-translator-icon\n\n|\n\n\n.. image:: https://img.shields.io/pypi/v/deep-translator.svg\n        :target: https://pypi.python.org/pypi/deep-translator\n.. image:: https://img.shields.io/travis/nidhaloff/deep-translator.svg\n        :target: https://github.com/nidhaloff/deep-translator/actions/workflows/build.yml\n.. image:: https://readthedocs.org/projects/deep-translator/badge/?version=latest\n        :target: https://deep-translator.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n.. image:: https://img.shields.io/pypi/l/deep-translator\n        :target: https://pypi.python.org/pypi/deep-translator\n\n.. image:: https://img.shields.io/pypi/status/deep-translator\n        :target: https://pypi.python.org/pypi/deep-translator\n\n.. image:: https://pepy.tech/badge/deep-translator\n    :target: https://pepy.tech/project/deep-translator\n\n\n.. image:: https://img.shields.io/pypi/wheel/deep-translator\n        :target: https://pypi.python.org/pypi/deep-translator\n\n.. image:: https://img.shields.io/twitter/url?url=https%3A%2F%2Ftwitter.com%2FNidhalBaccouri\n        :alt: Twitter URL\n        :target: https://twitter.com/NidhalBaccouri\n\n\n=======================\nTranslation for humans\n=======================\n\nA flexible **FREE** and **UNLIMITED** tool to translate between different languages in a simple way using multiple translators.\n\n\n* Free software: MIT license\n* Documentation: https://deep-translator.readthedocs.io.\n\n\n|\n|\n\n.. contents:: Table of Contents\n    :depth: 3\n\n|\n|\n\n\n==========\nMotivation\n==========\n\nI needed to translate a text using python. It was hard to find a simple way to do it.\nThere are other libraries that can be used for this task, but most of them\nare **buggy, not free, limited, not supported anymore or complex to use.**\n\nTherefore, I decided to build this simple tool. It is 100% free, unlimited, easy to use and provide\nsupport for all languages.\n\nBasically, my goal was to integrate support for multiple famous translators\nin this tool.\n\n======================\nWhen you should use it\n======================\n\n- If you want to translate text using python\n- If you want to translate from a file\n- If you want to get translations from many sources and not only one\n- If you want to automate translations\n- If you want to compare different translations\n- If you want to detect language automatically\n\n======================\nWhy you should use it\n======================\n\n- It\'s the only python tool that integrates many translators\n- multi language support\n- supports batch translation\n- High level of abstraction\n- Automatic language detection\n- Easy to use and extend\n- Support for most famous universal translators\n- Stable and maintained regularly\n- The API is very easy to use\n- Proxy integration is supported\n\n========\nFeatures\n========\n\n* Support for `google translate <https://translate.google.com/>`_\n* Support for the `microsoft translator <https://www.microsoft.com/en-us/translator//>`_ (version >= 1.3.5)\n* Support for `Pons translator <https://de.pons.com/>`_\n* Support for the `Linguee translator <https://www.linguee.com/>`_\n* Support for the `Mymemory translator <https://mymemory.translated.net/>`_\n* Support for the `Yandex translator <https://yandex.com/>`_ (version >= 1.2.1)\n* Support for the `QCRI translator <https://mt.qcri.org/api/>`_ (version >= 1.2.4)\n* Support for the `DeepL translator <https://www.deepl.com/en/translator/>`_ (version >= 1.2.5)\n* Support for the `Papago translator <https://papago.naver.com/>`_ (version >= 1.4.4)\n* Support for the `Libre translator <https://libretranslate.com/>`_\n* Support for proxy usage\n* Automatic single language detection\n* Batch language detection\n* Translate directly from a text file\n* Get multiple translation for a word\n* Automate the translation of different paragraphs in different languages\n* Translate directly from terminal (version >= 1.1.0)\n\n=============\nInstallation\n=============\n\nInstall the stable release:\n\n.. code-block:: console\n\n    $ pip install -U deep-translator\n\ntake a look at the docs if you want to install from source.\n\n============\nQuick Start\n============\n\n.. code-block:: python\n\n    from deep_translator import GoogleTranslator\n    translated = GoogleTranslator(source=\'auto\', target=\'de\').translate("keep it up, you are awesome")  # output -> Weiter so, du bist großartig\n\nor using proxies:\n\n.. code-block:: python\n\n    from deep_translator import GoogleTranslator\n\n    proxies_example = {\n        "https": "34.195.196.27:8080",\n        "http": "34.195.196.27:8080"\n    }\n    translated = GoogleTranslator(source=\'auto\', target=\'de\', proxies=proxies_example).translate("keep it up, you are awesome")  # output -> Weiter so, du bist großartig\n\n\nor even directly from terminal:\n\n.. code-block:: console\n\n    $ deep-translator translate -src "en" -tgt "de" -txt "keep it up, you are awesome"\n\n\n=====\nUsage\n=====\n\nIn this section, demos on how to use all different integrated translators in this tool are provided.\n\n.. note::\n\n    You can always pass the languages by the name or by abbreviation.\n\n    *Example*: If you want to use english as a source or target language, you can pass **english** or **en** as an argument\n\nImports\n--------\n\n.. code-block:: python\n\n    from deep_translator import (GoogleTranslator,\n                                 MicrosoftTranslator,\n                                 PonsTranslator,\n                                 LingueeTranslator,\n                                 MyMemoryTranslator,\n                                 YandexTranslator,\n                                 PapagoTranslator,\n                                 DeepL,\n                                 QCRI,\n                                 single_detection,\n                                 batch_detection)\n\n\nCheck Supported Languages\n---------------------------\n\n.. note::\n\n    You can check the supported languages of each translator by calling the\n    get_supported_languages function as a static method.\n\n.. code-block:: python\n\n    # default return type is a list\n    langs_list = GoogleTranslator.get_supported_languages()  # output: [arabic, french, english etc...]\n\n    # alternatively, you can the dictionary containing languages mapped to their abbreviation\n    langs_dict = GoogleTranslator.get_supported_languages(as_dict=True)  # output: {arabic: ar, french: fr, english:en etc...}\n\nLanguage Detection\n------------------\n\n.. note::\n\n    You can also detect language automatically. Notice that this package is free and my goal is to keep it free.\n    Therefore, you will need to get your own api_key if you want to use the language detection function.\n    I figured out you can get one for free here: https://detectlanguage.com/documentation\n\n- Single Text Detection\n\n.. code-block:: python\n\n    lang = single_detection(\'bonjour la vie\', api_key=\'your_api_key\')\n    print(lang) # output: fr\n\n- Batch Detection\n\n.. code-block:: python\n\n    lang = batch_detection([\'bonjour la vie\', \'hello world\'], api_key=\'your_api_key\')\n    print(lang) # output: [fr, en]\n\n\n\nGoogle Translate\n-----------------\n\n.. code-block:: python\n\n    text = \'happy coding\'\n\n- You can use automatic language detection to detect the source language:\n\n.. code-block:: python\n\n    translated = GoogleTranslator(source=\'auto\', target=\'de\').translate(text=text)\n\n- You can pass languages by name or by abbreviation:\n\n.. code-block:: python\n\n    translated = GoogleTranslator(source=\'auto\', target=\'german\').translate(text=text)\n\n    # Alternatively, you can pass languages by their abbreviation:\n    translated = GoogleTranslator(source=\'en\', target=\'de\').translate(text=text)\n\n- Translate batch of texts\n\n.. code-block:: python\n\n    texts = ["hallo welt", "guten morgen"]\n\n    # the translate_sentences function is deprecated, use the translate_batch function instead\n    translated = GoogleTranslator(\'de\', \'en\').translate_batch(texts)\n\n- Translate from a file:\n\n.. code-block:: python\n\n    translated = GoogleTranslator(source=\'auto\', target=\'german\').translate_file(\'path/to/file\')\n\nMymemory Translator\n--------------------\n\n.. note::\n\n    As in google translate, you can use the automatic language detection with mymemory by using "auto" as an\n    argument for the source language. However, this feature in the mymemory translator is not so powerful as\n    in google translate.\n\n- Simple translation\n\n.. code-block:: python\n\n    text = \'Keep it up. You are awesome\'\n\n    translated = MyMemoryTranslator(source=\'auto\', target=\'french\').translate(text)\n\n- Translate batch of texts\n\n.. code-block:: python\n\n    texts = ["hallo welt", "guten morgen"]\n\n    # the translate_sentences function is deprecated, use the translate_batch function instead\n    translated = MyMemoryTranslator(\'de\', \'en\').translate_batch(texts)\n\n- Translate from file\n\n.. code-block:: python\n\n    path = "your_file.txt"\n\n    translated = MyMemoryTranslator(source=\'en\', target=\'fr\').translate_file(path)\n\n\nDeepL Translator\n-----------------\n\n.. note::\n\n    In order to use the DeepL translator, you need to generate an api key. Deepl offers a Pro and a free API.\n    deep-translator supports both Pro and free APIs. Just check the examples below.\n    Visit https://www.deepl.com/en/docs-api/ for more information on how to generate your Deepl api key\n\n- Simple translation\n\n.. code-block:: python\n\n    text = \'Keep it up. You are awesome\'\n\n    translated = DeepL(api_key="your_api_key", source="en", target="en", use_free_api=True).translate(text)\n\n.. note::\n        deep-translator uses free deepl api by default. If you have the pro version then simply set the use_free_api to false.\n\n\n- Translate batch of texts\n\n.. code-block:: python\n\n    texts = ["hallo welt", "guten morgen"]\n\n    # the translate_sentences function is deprecated, use the translate_batch function instead\n    translated = DeepL("your_api_key").translate_batch(texts)\n\nQCRI Translator\n--------------------\n\n.. note::\n\n    In order to use the QCRI translator, you need to generate a free api key. Visit https://mt.qcri.org/api/\n    for more information\n\n- Check languages\n\n.. code-block:: python\n\n    # as a property\n    print("language pairs: ", QCRI("your_api_key").languages)\n\n- Check domains\n\n.. code-block:: python\n\n    # as a property\n    print("domains: ", QCRI("your_api_key").domains)\n\n- Text translation\n\n.. code-block:: python\n\n    text = \'Education is great\'\n\n    translated = QCRI("your_api_key").translate(source=\'en\', target=\'ar\', domain="news", text=text)\n    # output -> التعليم هو عظيم\n\n    # see docs for batch translation and more.\n\nLinguee Translator\n-------------------\n\n.. code-block:: python\n\n    word = \'good\'\n\n- Simple Translation\n\n.. code-block:: python\n\n    translated_word = LingueeTranslator(source=\'english\', target=\'french\').translate(word)\n\n    # pass language by their abbreviation\n    translated_word = LingueeTranslator(source=\'en\', target=\'fr\').translate(word)\n\n- Return all synonyms or words that matches\n\n.. code-block:: python\n\n    # set the argument return_all to True if you want to get all synonyms of the word to translate\n    translated_word = LingueeTranslator(source=\'english\', target=\'french\').translate(word, return_all=True)\n\n- Translate a batch of words\n\n.. code-block:: python\n\n    translated_words = LingueeTranslator(source=\'english\', target=\'french\').translate_words(["good", "awesome"])\n\nPONS Translator\n----------------\n\n.. note::\n\n    You can pass the languages by the name or by abbreviation just like\n    previous examples using GoogleTranslate\n\n.. code-block:: python\n\n    word = \'awesome\'\n\n- Simple Translation\n\n.. code-block:: python\n\n    translated_word = PonsTranslator(source=\'english\', target=\'french\').translate(word)\n\n    # pass language by their abbreviation\n    translated_word = PonsTranslator(source=\'en\', target=\'fr\').translate(word)\n\n- Return all synonyms or words that matches\n\n.. code-block:: python\n\n    # set the argument return_all to True if you want to get all synonyms of the word to translate\n    translated_word = PonsTranslator(source=\'english\', target=\'french\').translate(word, return_all=True)\n\n- Translate a batch of words\n\n.. code-block:: python\n\n    translated_words = LingueeTranslator(source=\'english\', target=\'french\').translate_words(["good", "awesome"])\n\nYandex Translator\n------------------\n\n.. note::\n\n    You need to require an **private api key** if you want to use the yandex translator.\n    visit the official website for more information about how to get one\n\n- Language detection\n\n.. code-block:: python\n\n    lang = YandexTranslator(\'your_api_key\').detect(\'Hallo, Welt\')\n    print(f"language detected: {lang}")  # output -> language detected: \'de\'\n\n- Text translation\n\n.. code-block:: python\n\n    # with auto detection | meaning provide only the target language and let yandex detect the source\n    translated = YandexTranslator(\'your_api_key\').translate(source="auto", target="en", text=\'Hallo, Welt\')\n    print(f"translated text: {translated}")  # output -> translated text: Hello world\n\n    # provide source and target language explicitly\n    translated = YandexTranslator(\'your_api_key\').translate(source="de", target="en", text=\'Hallo, Welt\')\n    print(f"translated text: {translated}")  # output -> translated text: Hello world\n\n- File translation\n\n.. code-block:: python\n\n    translated = YandexTranslator(\'your_api_key\').translate_file(source="auto", target="en", path="path_to_your_file")\n\n- Batch translation\n\n.. code-block:: python\n\n    translated = YandexTranslator(\'your_api_key\').translate_batch(source="auto", target="de", batch=["hello world", "happy coding"])\n\n\nMicrosoft Translator\n---------------------\n\n.. note::\n\n    You need to require an **api key** if you want to use the microsoft translator.\n    visit the official website for more information about how to get one.\n    Microsoft offers a free tier 0 subscription (2 million characters per month).\n\n- Required and optional attributes\n\n    There are two required attributes, namely "api_key" (string) and "target" (string or list).\n    Attribute "source" is optional.\n    Also, Microsoft API accepts a number of other optional attributes, you can find them here:  https://docs.microsoft.com/azure/cognitive-services/translator/reference/v3-0-translate\n    You can simply add them after the required attributes, see the example.\n\n.. code-block:: python\n\n    text = \'happy coding\'\n    translated = MicrosoftTranslator(api_key=\'some-key\', target=\'de\').translate(text=text)\n    translated_two_targets = MicrosoftTranslator(api_key=\'some-key\', target=[\'de\', \'ru\']).translate(text=text)\n    translated_with_optional_attr = MicrosoftTranslator(api_key=\'some-key\', target=\'de\', textType=\'html\']).translate(text=text)\n\n- You can pass languages by name or by abbreviation:\n\n.. code-block:: python\n\n    translated = MicrosoftTranslator(api_key=\'some-key\', target=\'german\').translate(text=text)\n\n    # Alternatively, you can pass languages by their abbreviation:\n    translated = MicrosoftTranslator(api_key=\'some-key\', target=\'de\').translate(text=text)\n\n- Translate batch of texts\n\n.. code-block:: python\n\n    texts = ["hallo welt", "guten morgen"]\n    translated = MicrosoftTranslator(api_key=\'some-key\', target=\'english\').translate_batch(texts)\n\n- Translate from a file:\n\n.. code-block:: python\n\n    translated = MicrosoftTranslator(api_key=\'some-key\', target=\'german\').translate_file(\'path/to/file\')\n\n\nPapago Translator\n---------------------\n\n.. note::\n\n    You need to require a **client id** and **client secret key** if you want to use the papago translator.\n    visit the official website for more information about how to get one.\n\n.. code-block:: python\n\n    text = \'happy coding\'\n    translated = PapagoTranslator(client_id=\'your_client_id\', secret_key=\'your_secret_key\', source=\'en\', target=\'ko\').translate(text=text)  # output: 행복한 부호화\n\n\nLibre Translator\n---------------------\n\n.. note::\n\n    Libre translate has multiple  `mirrors <https://github.com/LibreTranslate/LibreTranslate#mirrors>`_ which can be used for the API endpoint.\n    Some require an API key to be used. By default the base url is set to `libretranslate.de <https://libretranslate.de/>`_ . \n    This can be set using the "base_url" input parameter.\n\n.. code-block:: python\n\n    text = \'laufen\'\n    translated = LibreTranslator(source=\'auto\', target=\'en\', base_url = \'https://libretranslate.com/\', api_key = \'your_api_key\').translate(text=text)  # output: run\n\n\n- You can pass languages by name or by abbreviation:\n\n.. code-block:: python\n\n    translated = LibreTranslator(source=\'german\', target=\'english\').translate(text=text)\n\n    # Alternatively, you can pass languages by their abbreviation:\n    translated = LibreTranslator(source=\'de\', target=\'en\').translate(text=text)\n\n\n- Translate batch of texts\n\n.. code-block:: python\n\n    texts = ["hallo welt", "guten morgen"]\n    translated = LibreTranslator(source=\'auto\', target=\'en\').translate_batch(texts)\n\n- Translate from a file:\n\n.. code-block:: python\n\n    translated = LibreTranslator(source=\'auto\', target=\'en\').translate_file(\'path/to/file\')\n    \n    \n\nProxy usage\n-------------\n\ndeep-translator provide out of the box usage of proxies. Just define your proxies config as a dictionary\nand pass it to the corresponding translator. Below is an example using the GoogleTranslator but this feature\ncan be used with all supported translators.\n\n.. code-block:: python\n\n    from deep_translator import GoogleTranslator\n\n    # define your proxy configs:\n    proxies_example = {\n        "https": "your https proxy",  # example: 34.195.196.27:8080\n        "http": "your http proxy if available"\n    }\n    translated = GoogleTranslator(source=\'auto\', target=\'de\', proxies=proxies_example).translate("this package is awesome")\n\n\n\n\nUsage from Terminal\n--------------------\n\nDeep-translator supports a series of command line arguments for quick and simple access to the translators directly in your console.\n\n.. note::\n\n    The program accepts ``deep-translator`` or ``dt`` as a command, feel free to substitute whichever you prefer.\n\nFor a list of available translators:\n\n.. code-block:: console\n\n    $ deep-translator list\n\nTo translate a string or line of text:\n\n.. code-block:: console\n\n    $ deep_translator google --source "english" --target "german" --text "happy coding"\n\nAlternate short option names, along with using language abbreviations:\n\n.. code-block:: console\n\n    $ deep_translator google -src "en" -tgt "de" -txt "happy coding"\n\n\nFinally, to retrieve a list of available languages for a given translator:\n\n.. code-block:: console\n\n    $ deep-translator languages google\n\n======\nTests\n======\n\nDevelopers can install the development version of deep-translator and execute unit tests to verify functionality. For more information on doing this, see `the contribution guidelines <https://deep-translator.readthedocs.io/en/latest/contributing.html/>`_\n\n========\nLinks\n========\nCheck this article on medium to know why you should use the deep-translator package and how to translate text using python.\nhttps://medium.com/@nidhalbacc/how-to-translate-text-with-python-9d203139dcf5\n\n======\nHelp\n======\n\nIf you are facing any problems, please feel free to open an issue.\nAdditionally, you can make contact with the author for further information/questions.\n\nDo you like deep-translator?\nYou can always help the development of this project by:\n\n- Following on github and/or twitter\n- Promote the project (ex: by giving it a star on github)\n- Watch the github repo for new releases\n- Tweet about the package\n- Help others with issues on github\n- Create issues and pull requests\n- Sponsor the project\n\n==========\nNext Steps\n==========\n\nTake a look in the examples folder for more :)\nContributions are always welcome.\nRead the Contribution guidelines `Here <https://deep-translator.readthedocs.io/en/latest/contributing.html/>`_\n\n==========\nCredits\n==========\n\nMany thanks to @KirillSklyarenko for his work on integrating the microsoft translator\n\n==========\nLicense\n==========\n\nMIT license\n\nCopyright (c) 2020-present, Nidhal Baccouri\n\n\n===========================\nThe Translator++ mobile app\n===========================\n\n.. image:: ../assets/app-icon.png\n    :width: 100\n    :alt: Icon of the app\n\n\nAfter developing the deep-translator, I realized how cool this would be if I can use it as an app on my mobile phone.\nSure, there is google translate, pons and linguee apps etc.. but isn\'t it cooler to make an app where all these\ntranslators are integrated?\n\nLong story short, I started working on the app. I decided to use the `kivy framework <https://kivy.org/#home/>`_ since\nI wanted to code in python and to develop a cross platform app.\nI open sourced the `Translator++ app <https://github.com/nidhaloff/deep-translator-app/>`_ on my github too.\nFeel free to take a look at the code or make a pull request ;)\n\n.. note::\n    The Translator++ app is based on the deep-translator package. I just built the app to prove the capabilities\n    of the deep-translator package ;)\n\nI published the first release on google play store on 02-08-2020\n\nHere are some screenshots:\n\n- Phone\n\n.. image:: ../assets/translator1.jpg\n    :width: 30%\n    :height: 200\n    :alt: screenshot1\n.. image:: ../assets/translator2.jpg\n    :width: 30%\n    :height: 200\n    :alt: screenshot2\n.. image:: ../assets/spinner.jpg\n    :width: 30%\n    :height: 200\n    :alt: spinner\n\n- Tablet:\n\n.. image:: ../assets/hz_view.png\n    :width: 100%\n    :height: 300\n    :alt: screenshot3\n',
    'author': 'Nidhal Baccouri',
    'author_email': 'nidhalbacc@gmail.com',
    'maintainer': 'Nidhal Baccouri',
    'maintainer_email': 'nidhalbacc@gmail.com',
    'url': 'https://github.com/nidhaloff/deep_translator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import requests
from .constants import BASE_URLS, DEEPL_LANGUAGE_TO_CODE
from .exceptions import (ServerException,
                        TranslationNotFound,
                        LanguageNotSupportedException,
                        AuthorizationException)


class DeepL(object):
    """
    class that wraps functions, which use the DeepL translator under the hood to translate word(s)
    """
    _languages = DEEPL_LANGUAGE_TO_CODE

    def __init__(self, api_key=None, source="en", target="en", use_free_api=True, **kwargs):
        """
        @param api_key: your DeepL api key.
        Get one here: https://www.deepl.com/docs-api/accessing-the-api/
        @param source: source language
        @param target: target language
        """
        if not api_key:
            raise ServerException(401)
        self.version = 'v2'
        self.api_key = api_key
        self.source = self._map_language_to_code(source)
        self.target = self._map_language_to_code(target)
        if use_free_api:
            self.__base_url = BASE_URLS.get("DEEPL_FREE").format(version=self.version)
        else:
            self.__base_url = BASE_URLS.get("DEEPL").format(version=self.version)

    def translate(self, text, **kwargs):
        """
        @param text: text to translate
        @return: translated text
        """
        # Create the request parameters.
        translate_endpoint = 'translate'
        params = {
            "auth_key": self.api_key,
            "source_lang": self.source,
            "target_lang": self.target,
            "text": text
        }
        # Do the request and check the connection.
        try:
            response = requests.get(self.__base_url + translate_endpoint, params=params)
        except ConnectionError:
            raise ServerException(503)
        # If the answer is not success, raise server exception.
        if response.status_code == 403:
            raise AuthorizationException(self.api_key)
        elif response.status_code != 200:
            raise ServerException(response.status_code)
        # Get the response and check is not empty.
        res = response.json()
        if not res:
            raise TranslationNotFound(text)
        # Process and return the response.
        return res['translations'][0]['text']

    def translate_batch(self, batch, **kwargs):
        """
        @param batch: list of texts to translate
        @return: list of translations
        """
        return [self.translate(text, **kwargs) for text in batch]

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        return [*DeepL._languages.keys()] if not as_dict else DeepL._languages

    def _is_language_supported(self, lang, **kwargs):
        # The language is supported when is in the dicionary.
        return lang == 'auto' or lang in self._languages.keys() or lang in self._languages.values()

    def _map_language_to_code(self, lang, **kwargs):
        if lang in self._languages.keys():
            return self._languages[lang]
        elif lang in self._languages.values():
            return lang
        raise LanguageNotSupportedException(lang)


if __name__ == '__main__':
    d = DeepL(target="de")
    t = d.translate("I have no idea")
    print("text: ", t)



# -*- coding: utf-8 -*-

import requests
import logging
import sys

from .constants import BASE_URLS, MICROSOFT_CODES_TO_LANGUAGES
from .exceptions import LanguageNotSupportedException, ServerException, MicrosoftAPIerror


class MicrosoftTranslator:
    """
    the class that wraps functions, which use the Microsoft translator under the hood to translate word(s)
    """

    _languages = MICROSOFT_CODES_TO_LANGUAGES
    supported_languages = list(_languages.values())

    def __init__(self, api_key=None, region=None, source=None, target=None, proxies=None, **kwargs):
        """
        @params api_key and target are the required params
        @param api_key: your Microsoft API key
        @param region: your Microsoft Location
        """
        if not api_key:
            raise ServerException(401)
        else:
            self.api_key = api_key

        self.proxies = proxies
        self.headers = {
            "Ocp-Apim-Subscription-Key": self.api_key,
            "Content-type": "application/json",
        }
        # region is not required but very common and goes to headers if passed
        if region:
            self.region = region
            self.headers["Ocp-Apim-Subscription-Region"] = self.region

        if not target:
            raise ServerException(401)
        else:
            if type(target) is str:
                self.target = target.lower()
            else:
                self.target = [i.lower() for i in target]
            if self.is_language_supported(self.target):
                self.target = self._map_language_to_code(self.target)

        self.url_params = {'to': self.target, **kwargs}

        if source:
            self.source = source.lower()
            if self.is_language_supported(self.source):
                self.source = self._map_language_to_code(self.source)
            self.url_params['from'] = self.source

        self.__base_url = BASE_URLS.get("MICROSOFT_TRANSLATE")

    @staticmethod
    def get_supported_languages(as_dict=False, **kwargs):
        """
        return the languages supported by the microsoft translator
        @param as_dict: if True, the languages will be returned as a dictionary mapping languages to their abbreviations
        @return: list or dict
        """
        return MicrosoftTranslator.supported_languages if not as_dict else MicrosoftTranslator._languages

    def _map_language_to_code(self, language, **kwargs):
        """
        map the language to its corresponding code (abbreviation) if the language was passed by its full name by the user
        @param language: a string (if 1 lang) or a list (if multiple langs)
        @return: mapped value of the language or raise an exception if the language is not supported
        """
        if type(language) is str:
            language = [language]
        for lang in language:
            if lang in self._languages.values():
                yield lang
            elif lang in self._languages.keys():
                yield self._languages[lang]
            else:
                raise LanguageNotSupportedException(lang)

    def is_language_supported(self, language, **kwargs):
        """
        check if the language is supported by the translator
        @param language: a string (if 1 lang) or a list (if multiple langs)
        @return: bool or raise an Exception
        """
        if type(language) is str:
            language = [language]
        for lang in language:
            if lang not in self._languages.keys():
                if lang not in self._languages.values():
                    raise LanguageNotSupportedException(lang)
        return True

    def translate(self, text, **kwargs):
        """
        function that uses microsoft translate to translate a text
        @param text: desired text to translate
        @return: str: translated text
        """
        # a body must be a list of dicts to process multiple texts;
        # I have not added multiple text processing here since it is covered by the translate_batch method
        valid_microsoft_json = [{'text': text}]
        try:
            requested = requests.post(self.__base_url,
                                      params=self.url_params,
                                      headers=self.headers,
                                      json=valid_microsoft_json,
                                      proxies=self.proxies)
        except requests.exceptions.RequestException:
            exc_type, value, traceback = sys.exc_info()
            logging.warning(f"Returned error: {exc_type.__name__}")

        # Where Microsoft API responds with an api error, it returns a dict in response.json()
        if type(requested.json()) is dict:
            error_message = requested.json()['error']
            raise MicrosoftAPIerror(error_message)
        # Where it responds with a translation, its response.json() is a list e.g. [{'translations': [{'text': 'Hello world!', 'to': 'en'}]}]
        elif type(requested.json()) is list:
            all_translations = [i['text'] for i in requested.json()[0]['translations']]
            return "\n".join(all_translations)

    def translate_file(self, path, **kwargs):
        """
        translate from a file
        @param path: path to file
        @return: translated text
        """
        try:
            with open(path, 'r', encoding='utf-8') as f:
                text = f.read().strip()
            return self.translate(text)
        except Exception as e:
            raise e

    def translate_batch(self, batch, **kwargs):
        """
        translate a batch of texts
        @param batch: list of texts to translate
        @return: list of translations
        """
        return [self.translate(text, **kwargs) for text in batch]
